\ RTX Disassembler                                   Jon 16/06/89
\ Version 3 - October 1990

((
  (C) MicroProcessor Engineering & Harris Semiconductor Corporation

  MicroProcessor Engineering
  133 Hill Lane
  Southampton SO1 5AF
  England

  Tel: +44 703 631441
  Fax: +44 703 339691

  Takes primitive from Classes 8 - 15 and displays equivalent
  Forth mnemonics.

  Based on RTX disassembler from Harris Semiconductor Corp.

  1/Nov/90 - Conversion to RTXUF in module 3, DECOMP.XS3. Version number
  upgraded to v3.0
))



\ Variables & Constants .................          Jon 09/08/89

variable Last_Literal			\ holds value of last literal
variable Streamed?			\ true for streamed instruction
variable at-end				\ true if last return detected
variable (in-colon?)			\ true if inside colon definition
variable max-addr			\ holds highest forward branch addr

decimal

60 constant page-length



\ Misc words

hex

: .ASCII	\ char -- ; display printables as is, or display '.'
  dup 020 07E within? 0=
  if  drop  ascii .  endif
  emit  ;

: .byte		\ b -- ; display as two hex digits
  base @ swap  hex
  0 <# # # #> type
  base !  ;

: .word		\ n -- ; display as four hex digits
  base @ swap  hex
  0 <# # # # # #> type
  base !  ;

: WORD->ASCII	\ n -- ; display word as two ASCII characters.
  ASCII ' EMIT  
  0 0100 um/mod .ascii .ascii
  ASCII ' EMIT  ;

: SET-IN-COLON	\ -- ; set flag
  -1 (IN-COLON?) !  ;

: .hex-addr	\ addr page -- ; display address as x:yyyy
   .byte ." :" .word  ;

: .DASM_ADDR	\ addr -- ; show address for current page
   ( DASM_PAGE @ ) 0		\ for the moment use page 0
   .hex-addr  ;

\ Display a substring of a counted "superstring"
: .ENTRY	\ i addr -- ; display ith (0..i) substring
  SWAP 0			\ for i strings
  ?DO  count +  LOOP		\ skip over each one
  COUNT TYPE  ;			\ and then display

\ Returns flag indicating whether we are in the middle of a
\  colon definition.
: IN-COLON?	\ -- f
  (IN-COLON?) @ 0<>  ;
\ Counted Super Strings - Stack manipulation       sfp 03/04/90

here
 ", "                     \ NOP       {inv} shift
 ", drop  dup  "          \ DROP DUP  {inv} shift
 ", over swap  "          \ UNDER      alu  shift
 ", swap drop  "          \ NIP       {inv} shift
 ", drop  "               \ DROP      {inv} shift
 ", "                     \ NOP        alu  shift
 ", swap drop  dup  "     \ NIP DUP   {inv} shift
 ", swap "                \ SWAP      {inv} shift
 ", swap over  "          \ TUCK       alu  shift
 ", dup "                 \ DUP       {inv} shift
 ", over "                \ OVER      {inv} shift
 ", 2dup "                \ 2DUP       alu  shift
Constant "A-class"


\ Counted Super Strings - Shifts ........          sfp 03/04/90

here
 ", "            ", 0< "
 ", 2* "         ", 2*c "
 ", cU2/ "       ", c2/ "
 ", U2/ "        ", 2/ "
 ", N2* "        ", N2*c "
 ", D2* "        ", D2*c "
 ", cUD2/ "      ", cD2/ "
 ", UD2/ "       ", D2/ "
Constant "ssss"


\ Counted Super Strings - math table - cccc        sfp 03/04/90

here
 ", "                 ", not "
 ", and "             ", nor "
 ", swap -  "         ", swap -c  "
 ", or "              ", nand "
 ", + "               ", +c "
 ", xor "             ", xnor "
 ", - "               ", -c "
 ",  "                ", not "
Constant "cccc"


\ Counted Super Strings - brnch tables ..          Jon 15/08/89

here
 ", ?dup 0branch  "
 ", 0branch "
 ", branch "
 ", next "
Constant "branch"
\ Counted Super Strings - G-bus operations         sfp 03/04/90

here
\  read       read ;          write            write ;
 ", r@ "     ", r@ "       ", ir! ( r> drop >r )  "  ", ; >r  "
 ", r> "     ", r@ "       ", >r  "          ", ; >r  "
 ", r@ 2*  " ", r@ 2*  "   ", Of( "          ", ; Of( "
 ", cr@ "    ", cr@ "      ", cr!  "         ", cr!  "
 ", md@ "    ", md@ "      ", md!  "         ", md!  "
 ", sq@ "    ", sq@ "      ", 256 * md!  "   ", 256 * md!  "
 ", sr@ "    ", sr@ "      ", sr!  "         ", sr!  "
 ", pc@ "    ", pc@ "      ", execute "      ", >r  "
Constant "core-regs"


\ Counted Super Strings - internal G-bus ops        \  14/01/90

here
 ", imr"        ", spr"
 ", 0A g"       ", ivr/slr"
 ", ipr"        ", dpr"
 ", upr"        ", cpr"
 ", ibc"        ", ubr"
 ", 012 g"      ", tc0"
 ", tc1"        ", tc2"
 ", mrl"        ", mrh"
Constant "int-g"


\ Counted Super Strings - regs & s-lits            sfp 03/04/90

here
 ", drop  "          \ g G@  DROP  {inv}
 ", "                \ g G@        {inv}
 ", over "           \ g G@  OVER  alu-op
 ", drop  "          \ d DROP   {inv}
 ", "                \ d        {inv}
 ", over "           \ d OVER alu-op
 ", "                \ DUP g G! {inv}
 ", "                \     g G! {inv}
 ", "                \ g G@ SWAP alu-op
 ", undefined "      \
 ", swap drop  "     \ d SWAP DROP {inv}
 ", "                \ d SWAP alu-op
Constant "reg&sh-lit"

\ Counted Super Strings - memory by words          sfp 03/04/90

here
 ", @ swap  "          \ @ SWAP              | {inv}
 ", @  "               \ @ SWAP              | SWAP {inv}
 ", @ over  "          \ @ SWAP              | SWAP OVER alu
 ", dup @ swap  "      \ {NIP} DUP @ SWAP    | nop
 ", @  "               \ {NIP} @ d           | nop
 ", dup @ swap  "      \ {NIP} DUP @ SWAP d SWAP alu | nop
 ", over swap !  "     \ OVER SWAP !         | {inv}
 ", !  "               \ OVER SWAP !         | DROP {inv}
 ", @  "               \ @ SWAP              | alu-op
 ", swap over !  "     \ {UNDER} SWAP OVER ! | nop
 ", !  "               \ {UNDER}  ! d        | nop
 ", swap over !  "     \ {UNDER} SWAP OVER ! d SWAP alu |nop
Constant "<#mem"

\ Counted Super Strings - memory by bytes          sfp 03/04/90

here
 ", c@ swap  "           \ C@ SWAP              | {inv}
 ", c@  "                \ C@ SWAP              | SWAP {inv}
 ", c@ over  "           \ C@ SWAP              | SWAP OVER alu
 ", dup c@ swap  "       \ {NIP} DUP C@ SWAP    | nop
 ", c@  "                \ {NIP} C@ d           | nop
 ", dup c@ swap  "       \ {NIP} DUP C@ SWAP d SWAP alu | nop
 ", over swap c!  "      \ OVER SWAP C!         | {inv}
 ", c!  "                \ OVER SWAP C!         | DROP {inv}
 ", c@  "                \ C@ SWAP              | alu-op
 ", swap over c!  "      \ {UNDER} SWAP OVER C! | nop
 ", c!  "                \ {UNDER}  C! d        | nop
 ", swap over c!  "      \ {UNDER} SWAP OVER C! d SWAP alu |nop
Constant "<#memc"


\ Counted Super Strings - user space               sfp 03/04/90

here
 ", u@ swap  "        \ u U@ SWAP  | {inv}
 ", u@  "             \ u U@ SWAP  | SWAP {inv}  (U@)
 ", u@ over  "        \ u U@ SWAP  | SWAP OVER alu-op
 ", u!  "             \ DUP u U!   | {inv}
 ", u!  "             \ DUP u U!   | DROP {inv}  (U!)
 ", u@  "             \ u U@ SWAP  | alu-op
Constant "user#>"


\ Counted Super Strings - long literals            sfp 03/04/90

here
 ", swap "           \ D SWAP    | {inv}
 ", "                \ D SWAP    | SWAP {inv}
 ", over "           \ D SWAP    | SWAP OVER  alu-op
 ", undefined "
 ", undefined "
 ", undefined "
 ", swap "           \ D SWAP    | {inv}
 ", swap drop  "     \ D SWAP    | DROP {inv}
 ", "                \ D SWAP    | alu-op
 ", undefined "
 ", undefined "
 ", undefined "
Constant "lit"

\ Counted Super Strings - 17 bit math              Jon 15/08/89

here
", md@ "    ", md@ 2* sr@ or  "
Constant "r@"

here
", "       ", 2*' "    ", c2/' "   ", 2/' "
", n2*' "  ", d2*' "   ", cD2/' "  ", d2/' "
Constant "shift17"

here
", dup "       ", over and  "  ", over swap -  "
", over or  "  ", over +  "    ", over xor  "
", over -  "   ", "
Constant "alu17"



\ Special definitions                            16 \  14/01/90
\ Opcodes which represent standard FORTH words but which would
\ disassemble into several primitives. MEANS" Builds code table.
\   Usage     n means" string"
\ : Means"   ,  ( code)  ",  ;  \ not supported by XC

hex

create Means-Table
a000 ,   ", nop "       bf40 ,   ", lit -1 "
a0C1 ,   ", s>d "       e0c0 ,   ", 2dup !  "
f0c0 ,   ", 2dup c!  "
bcc0 ,   ", negate "    b4c1 ,   ", 1- "
b08d ,   ", seldpr "    b00d ,   ", selcpr "
b090 ,   ", softint "   b010 ,   ", -softint "
b096 ,   ", mulu "      b097 ,   ", muls "
0000 ,          \ End of table


\ MEANS?                                            \  14/01/90

hex

\ Scans the codes contained in MEANS-TABLE to determine if the
\ current one is a standard TFORTH word. Returns true if it is.
: Means?	\ opcode -- f 
  dup 0a020 =				\ return?
  If  drop 0 exit Then      		\ Return is special case
  0ffdf and  >r                         \ strip return bit
  Means-Table                           \
  Begin  dup @  ?dup                    \ 0 = table end
    While  r@ =                         \ opcode in table ?
           If  2+ count type            \ display menemonic(s)
               r> drop  -1 exit         \ get out with flag
           Endif
           2+ count +			\ Bump to next entry
  Repeat                                \
  drop  r> drop  0  ;                   \ Clean up, & No match



\ Extract bit fields from instructions              \  13/01/90

hex

: Inv?       ( ins--f)  0100 and ;              \ Inverse bit
: Return?    ( ins--f)  0020 and 20 / ;         \ Return bit
: Ins-Group  ( ins--n)  00C0 and  40 / ;        \ 0-3, subgroup
: Shift-Bits ( ins--n)  000F and ;
: Lit-Bit    ( ins--f)  0040 and 40 / ;         \ Literal bit
: Data-Field ( ins--n)  001F and ;              \ G addr or LIT
: Nip?       ( ins--f)  inv? 0100 / negate ;    \ NIP/UNDER bit
: @?         ( ins--f)  dup 6 <  swap 8 =  or ; \ @ or !

decimal
\ ALU-BITS                                       08 \  14/01/90

HEX

\ Tests for Minus operators and converts to swapped form if the
\   instruction has an implicit SWAP in it.
: ALU-BITS      \ ins -- n
  DUP  8 >>n  000F AND
  DUP 4 =  OVER 5 = OR  OVER 0C = OR  OVER 0D = OR  \ Minus op?
  IF  SWAP  DUP 00C0 AND  SWAP F000 AND
      CASE  A000 OF  0000 =           ENDOF
            B000 OF  0080 AND 0080 =  ENDOF
            C000 OF  0000 <>          ENDOF
            E000 OF  0040 AND 0040 =  ENDOF
            F000 OF  0040 AND 0040 =  ENDOF
      ENDCASE
      IF  8 XOR THEN
  ELSE  NIP  THEN  ;


\ Extract index                                0 10Jon 16/06/89

HEX

\ Each subgroup of instructions may have 3 instructions:
\  ALU field = 000i, ALU field = 111i, or ALU field = ALU code
: GROUP-MEMBER  ( ALU field -- index)
  DUP 2 <
   IF   DROP  0          \ 0 = 000i
   ELSE 0D >             \ 1 = 111i
     IF 1 ELSE 2 THEN    \ 2 = ALU code
   THEN ;

\ gives ordinal number to opcode in class
: INS-INDEX  ( ins -- n)
  DUP ALU-BITS GROUP-MEMBER SWAP INS-GROUP 3 * + ;

 DECIMAL


\ Print data fields                                sfp 18/04/90

: .DATA         \ o --
  DATA-FIELD .word space ; \ display sh-lit data

: .SHORT-DATA   \ ins -- ; Prints short lit/G-addresses
  DATA-FIELD  DUP  LAST_LITERAL !  .byte space  ;

: .ALU-OP       \ o --
  ALU-BITS "cccc" .ENTRY ;              \ Displays math oper.

\ Lower 3 bits of some instructions are put out onto the ASIC
\  address bus. Print those bits.
: .XTENSION     \ ins --
   DATA-FIELD ?DUP
   IF  ." (GA = " 2 u.r ." )"  THEN ;

\ .END-OF-WORD  END-OF-WORD?                     16:37 03/24/89

HEX

\ Determines if the current instruction is the last one in a
\  Forth word by checking the address of the following
\  instruction. If the address points to the beginning of a
\  new word or past the end of memory being DASM'ed, this instr
\  is assumed to be the last one in the word.
: END-OF-WORD?	( addr inst -- f)
  0F000 AND  0D000 =			\ Inst = Long literal ?
  IF  4  ELSE  2  THEN			\ Offset to next instruction
  +  
  max-addr @ u>    			\ if no forward branch outstanding
  dup at-end !  ;			\ and save flag

: .END-OF-WORD
  ." ; "    in-colon?
  IF  0 (IN-COLON?) !   THEN  ;


\ Determines if return bit in word indicates end of definition
\  or EXIT
: .RETURN	\ addr instr --
  DUP RETURN?				\ Is return bit set?
  IF  END-OF-WORD?			\ Is next address a : def
      IF   .END-OF-WORD			\ Yes, so ; is a return
      ELSE ." EXIT"			\ No, is still in defin.
      ENDIF
  ELSE  2DROP				\ Return bit not set
  ENDIF  ;
\ BRANCH_DEST                                    07sfp 13/01/90

HEX

: BRANCH_DEST	\ addr instr -- addr'
  >R  2+                                     \ IP has been incremented
  FC00 AND                                   \ current block
  R@ 0600 AND                                \ "new block" bits
  CASE                                       \ Calculate new blk
    0000 OF   0000 +     ENDOF               \   Same block
    0200 OF   0400 +     ENDOF               \   Next block
    0400 OF   0000 AND   ENDOF               \   Block 0
    0600 OF   FC00 +     ENDOF               \   Previous block
    ENDCASE
  R>  01FF AND   2*                          \ Offset in new blk
  OR                                         \ Merge blk & offst
  dup max-addr u>                            \ store highest address
  if  dup max-addr !  endif  ;               \ to determine end of word
  


\ Branches                                       08sfp 13/01/90

hex

: ?"BRNCH"	( n -- n' )
  01800 AND  0800 /  ;			\ extract index into string

: .BRANCH	( addr instr -- inc )
  DUP  ?"BRNCH"  "branch" .ENTRY
  BRANCH_DEST .word
  2   ;

\ G-Space                                           \  14/01/90

HEX

: @_OR_!        \ instr -- f ; return 0 for @ or 1 for ! op.
  INS-INDEX  DUP 6 =  SWAP  7 =  OR
  IF  1  ELSE  0  THEN  ;

: "!/@"         \ instr -- ; Prints char. for @ or ! operation
  @_OR_!
  IF  ." !  "  ELSE  ." @ " THEN  ;

: EXTERNAL-G    \ instr -- instr)
  DUP  DATA-FIELD  .SHORT-DATA
  DUP  ." g"  "!/@" ;


hex

\ Prints name of core register. Strips return bit from
\  ; >R and ; OF( instructions so it won't be printed twice.
\ Sets "streaming" flag for OF( instruction
: .CORE-REGISTER	( instr -- instr' )
  DUP  DATA-FIELD  2 =  OVER  @_OR_!  AND  STREAMED? !
  DUP  DATA-FIELD  4 *   OVER @_OR_! 2* +   OVER RETURN? +
  "core-regs" .ENTRY
  DUP  DATA-FIELD  2 <=  OVER  @_OR_! AND    \ Write to 0-2?
   IF FFDF AND THEN ;                        \ Strip ; bit

: INTERNAL-G		( instr -- instr')
\ DUP  BE8F = IF  LAST_LITERAL @   CALL_PAGE !  THEN    \ CPR!
  DUP  DATA-FIELD
  DUP  7 >
  IF    8 -  "int-g" .ENTRY  DUP "!/@"
  ELSE  DROP .CORE-REGISTER
  THEN  ;

: G-SPACE	( instr -- instr)
  DUP  0EC0 AND 0080  = IF  ." DUP "  THEN
  DUP  DATA-FIELD  17 >
    IF   EXTERNAL-G
    ELSE INTERNAL-G   THEN ;

DECIMAL



\ Short literal/Registers                        12 \  14/01/90

\ Disassemble short literal instructions
: SHORT-LITERAL	( instr -- instr)
  DUP .SHORT-DATA ;

\ Disassemble register & short literal opcode
: REG&SH-LIT	( addr instr -- inc)
  DUP LIT-BIT  IF  SHORT-LITERAL  ELSE  G-SPACE  THEN
  DUP  INS-INDEX  "reg&sh-lit" .ENTRY
  dup .ALU-OP
  .return
  2 ( increment) ;


\ Long literal                                     sfp 18/04/90

: LONG-LITERAL  ( addr inst --  inc )
  ." lit " OVER ( addr)  2+ @(t) .word space
  DUP INS-INDEX "lit" .ENTRY
  DUP .ALU-OP
  2dup .return
  .XTENSION
  drop
  4   ( increment ) ;

\ Memory access                                1 15 \  14/01/90

HEX

\ Returns flag: TRUE = word instruction, FALSE = byte
: WORD? (  -- f)  1000 AND  1000 <> ;

\ NIP/UNDER?  determines whether a {nip/under} is present, and
\   prints the appropriate message, if so
: NIP/UNDER? ( instr -- )
  DUP 0100 AND  0100 =  IF DROP  EXIT  THEN   \ nip bit = 1
  0080 AND  0= IF    ." swap drop  "          \ NIP
               ELSE  ." over swap "           \ UNDER
               THEN ;

 DECIMAL


\ MEM-W/O-DATA                                   11Jon 16/06/89

: MEM-W/O-DATA ( addr instr -- )
  DUP INS-INDEX
  OVER  WORD?
  IF  "<#mem"  ELSE  "<#memc"  THEN
  .ENTRY
  DUP .ALU-OP
  2dup .return
  .XTENSION
  ( addr)  DROP  ;


\ Memory access                                1 12sfp 13/01/90

HEX

: MEM-W-DATA ( instr --)
  DUP NIP/UNDER?                   \ Check nip/under bit
  DUP INS-INDEX   OVER WORD?
    IF  "<#mem"  ELSE  "<#memc"  THEN     .ENTRY
  DUP ALU-BITS   0E AND  DUP
    0<> IF   OVER  ."   lit " .SHORT-DATA  THEN
   DUP  0<>  OVER  0E <>  AND
     IF  "cccc" .ENTRY   ELSE  DROP  THEN
  .return  ;

DECIMAL


\ Memory access                                  12sfp 13/01/90

: MEMORY ( addr inst -- inc )
  DUP LIT-BIT
    IF    MEM-W-DATA           \ Instructions with short literal
    ELSE  MEM-W/O-DATA         \ Straight fetch/store
    THEN
  2  ( increment)  ;
\ USER instructions                              15sfp 13/01/90

\ Tests for  DUP U! {inv} case  1100 000i 10;u uuuu
\  (ins-index = 6)
: .DUP? ( i)   6 = IF  ." DUP "  THEN ;

: USER-SPACE	( addr ins -- inc )
  DUP LIT-BIT                 \ LIT-BIT set = undefined code
  IF    2DROP ." undefined "
  ELSE  DUP INS-INDEX  DUP .DUP?
        OVER  ( ins) .SHORT-DATA        \ U space address
        ( index)  DUP 5 >
        If  3 - THEN
        "user#>" .ENTRY
        dup .ALU-OP
        .return
  THEN
  2  ( increment) ;
\ 17 bit math                                  10:0Jon 16/06/89

hex

\ If bit 0 of alu field = 0 (SWAP-,+,-) use CO16 for flag;
\ otherwise use bit 0 of instruction
: .AOP ( instr)
   DUP 200 / 1 AND  IF  1 AND .  ELSE  ." CO16 " DROP  THEN ;

: .YY ( instr)
  DUP   40 /  3 AND   ( yy field)
  CASE   ( instr)
    0 OF .AOP                                       ENDOF
    1 OF .AOP  ." CCQ OR "                          ENDOF
    2 OF  8 AND  0= IF ." T0 "  ELSE ." N0 "  THEN  ENDOF
    3 OF  DROP  ." N0 T0 XOR "                      ENDOF
    ENDCASE  ;

HEX

: 17-BIT-MATH   \ addr instr --
  DUP 100 /  0F AND  ( aaar field)
  DUP 2/  IF  DUP  1 AND "r@" .ENTRY THEN
  2/  ( alu) "alu17" .ENTRY
  DUP .YY     ." SWAP? DROP  "
  DUP 2/ 7 AND  "shift17" .ENTRY
  .return  ;


\ Math                                           14Jon 16/06/89

HEX

: .SHIFT  ( instr --)  SHIFT-BITS "ssss" .ENTRY ;

\ Is this a 17-bit math instruction?
: 17-BIT? ( instr -- f)  0010 AND  0010 / ;

: MATH ( addr instr -- inc )
  DUP 17-BIT?
  If    17-BIT-MATH
  Else  DUP INS-INDEX  "A-class" .ENTRY
         dup .ALU-OP  dup .SHIFT  .return
  Endif
  2  ;

\ Forms control

decimal

: .header
  line# off
  cr
  cr ." RTX 2000 Family Disassembler  V3.0-Beta" 
  cr ." Address   Opcode  Operations"
  cr  ;

: .footer
  base @  decimal
  cr ." Page " page# @ .
  base !  ;

\ Checks for bottom of page.
: NEW-LINE	\ --
  CR  LINE# @ PAGE-LENGTH u>
  IF  .FOOTER
      12 emit  1 page# +!
      .header
  ENDIF  ;



\ .symbol-name 

hex

: .symbol-name	\ segment offset -- ; display name from given pointer
   2dup c@l 01F and 0			\ for each character in name
   ?do  1+  2dup c@l			\ collect character
        dup ascii A ascii Z within?	\ if upper case
	if  020 or  endif		\  convert to lower case
        emit				\ and display
   loop					\
   2drop  ;				\ discard seg/offset

: COLON?	\ addr -- 
  CFA>NAME
  IF  ." : "  .symbol-name  NEW-LINE
      1 (IN-COLON?) !
  else  2drop
  THEN  ;

decimal

: SUBROUTINE	\ addr instr  -- inc
  dup 2* CFA>NAME			\ -- addr instr seg/offset t/f
  if  .symbol-name			\ found, display name
      43 OUT @ -  0 max  SPACES		\ and push rest out as comment
  else  2drop
  endif
  DUP 2*  ." Call " .word		\ show as call to address
  3 SPACES WORD->ASCII			\ and as two ASCII characters
  DROP ( addr )
  2  ( increment )  ;

((
\ UCODE?
\ Determines if the current instruction is a UCODE definition.
: UCODE? ( instr-- )
  ?NAMES @ IF
    UCODE>NAME  IF dbuff COUNT  TYPE  ."    UCODE:  "  THEN
  ELSE  DROP  THEN ;
))
\ INSTR-TYPE                                     07sfp 13/01/90
\ Determines type of instruction. Returns flag indicating class.
\ 0 = primitive   MEANS? prints the name
\ 7 = subroutine call
\ 8-15 = instruction to be decoded

decimal
((
\
\ This version does not disassemble outside a colon definition
\
: INSTR-TYPE	\ addr instr -- addr instr index
  IN-COLON?  IF
  DUP MEANS?
      IF   2DUP .RETURN  2 SPACES  0
      ELSE  DUP  12 >>n  7 MAX		\ < 8 is a call
      THEN
  ELSE  ." constant  "
        DUP .word  3 SPACES  DUP WORD->ASCII  0
  THEN  ;
))

\
\ This version disassembles under all conditions
\
: INSTR-TYPE	\ addr instr -- addr instr index
  DUP MEANS?
  IF   2DUP .RETURN  2 SPACES  0
  ELSE  DUP  12 >>n  7 MAX		\ < 8 is a call
  THEN  ;

hex

: process-instr	\ addr instr -- inc
  Case  instr-type
     0 Of  2drop 2          Endof
     7 Of  Subroutine       Endof
     8 Of  .branch          Endof
     9 Of  .branch          Endof
    0a Of  math             Endof
    0b Of  Reg&Sh-Lit       Endof
    0c Of  user-space       Endof
    0d Of  long-literal     Endof
    0e Of  memory           Endof
    0f Of  memory           Endof
  EndCase  ;
\ getting to [DASM]

decimal

\ Sets up global DASM parameters
: INIT-DASM	\ -- ; initialisation
  STREAMED? off  at-end off		\ turn off flags
  line# @ page-length u>		\ if over page end
  if  line# off  endif			\  assume starting fresh
  new-line  1 page# !  .header  ;	\ initialise print-out

: exit-dasm	\ -- ; close down the output
  NEW-LINE  12 emit  1 page# +!  ;

: [DASM]	\ addr length --
  >api-window				\ switch input and output
  BASE @ >R  HEX			\ work in hex
  over max-addr !			\ last branch target
  INIT-DASM				\ initialise
  (in-colon?) off			\ assume NOT in colon definition
  bounds
  ?DO  in-colon? not
       if  i colon?  endif
       I DUP @(t)
       OVER .DASM_ADDR  3 SPACES
       DUP .word
       STREAMED? @
       IF  ."  s  "
       ELSE  4 SPACES
       THEN
       0 STREAMED? !
\      DUP UCODE?
       ( addr inst ) PROCESS-INSTR
       NEW-LINE
       halt?
       IF  DROP  LEAVE  THEN
  +LOOP
  exit-dasm
  R> BASE !  ;

: [dis]		\ addr -- ; decompile word at addr
  >api-window				\ switch input and output
  BASE @ >R  HEX			\ work in hex
  dup max-addr !			\ last branch target
  INIT-DASM				\ initialise
  (IN-COLON?) on 			\ assume in : definition.
  dup colon?				\ display name if definition found
  $01000 bounds
  ?DO  I DUP @(t)
       OVER .DASM_ADDR  3 SPACES
       DUP .word
       STREAMED? @
       IF  ."  s  "
       ELSE  4 SPACES
       THEN
       0 STREAMED? !
\      DUP UCODE?
       ( addr inst ) PROCESS-INSTR
       NEW-LINE
       at-end @ 0<>  halt? or
       IF  DROP  LEAVE  THEN
  +LOOP
  exit-dasm
  R> BASE !  ;
\ the end


;p
