((
        RTXDBIO.FTH

        Author: J Lee

        Date started:   05/12/88
        Last modified:  13/08/91


        Harris RTXDB I/O handlers

        Copyright (C) 1988, 1989, 1990 MicroProcessor Engineering

        Released for use with the MPE RTX 2000 Forth Cross Compiler by:

        MicroProcessor Engineering
        133 Hill Lane
        Shirley
        Southampton SO1 5AF
        England

        tel: (+44) 703 631441
))




\ RTX-2000 IO Loading .. JL 5/12/89
\

Only Forth Definitions



\ UART driver - constants .. JL 5/12/89
\

hex

018 constant Port0   \ Parallel port 0 = 018h



\ bit masks for generating control words
\
00000 constant Quiet    \ all control lines inactive
01000 constant ReadT    \ DISTR (bit 12) active
0efff constant ReadF    \ DISTR inactive
02000 constant WriteT   \ DOSTR (bit 13) active
0dfff constant WriteF   \ DOSTR inactive



\ register masks .. JL 5/12/89
\

\ bit masks for register addresses (bits 9-11). These masks have the
\ Chip Select bit (bit 8) set

hex

0100 constant Rbr   \ receive buffer address
0100 constant Thr   \ transmitter holding register
0300 constant Ier   \ interrupt enable register
0500 constant Iir   \ interrupt identification register
0700 constant Lcr   \ line control register
0900 constant Mcr   \ modem control register
0b00 constant Lsr   \ line status register
0d00 constant Msr   \ modem status register
0f00 constant Scr   \ scratch register
0100 constant Dll   \ divisor latch LSB
0300 constant Dlm   \ divisor latch MSB




\ Reg@ .. JL 5/12/89
\

hex

: Reg@   ( reg# -- data )
  quiet  ( port0 ) 018 g!		\ quiet pattern
  dup  ( port0 ) 018 g!			\ select UART register and CS
  noop					\ wait 1 PCLK cycle
  readt or  dup ( port0 ) 018 g!	\ set DISTR true
  noop noop noop			\ wait 3 PCLK cycles
  ( port0 ) 018 g@  0ff and		\ read data from in port 0
  swap  readf and  ( port0 ) 018 g!	\ set DISTR false
  quiet ( port0 ) 018 g!		\ quiet pattern
  noop  ;



\ Reg! ..  JL 5/12/89
\

: Reg!   ( data reg# -- )
  quiet ( port0 ) 018 g!		\ quiet pattern
  dup ( port0 ) 018 g!			\ select UART register
  noop
  or					\ combine data, sel&addr
  writet or				\ set write bit
  dup ( port0 ) 018 g!			\ set DOSTR true
  noop noop
  writef and  ( port0 ) 018 g!		\ set DOSTR false
  noop
  quiet ( port0 ) 018 g!		\ quiet pattern
  noop  ;



\ Reset-UART .. JL 5/12/89
\

hex

: Reset-UART   ( -- )
  080 Lcr reg!   \ set DLAB bit
  004 Dll reg!   \ set baud to 9600
  000 Dlm reg!   \ 2.4576MHz / 16(010h) ==> 9600 baud
  003 Lcr reg!   \ 8-data, 1-stop, no parity
  000 Mcr reg!   \ disable OUT1, OUT2; clear RTS, CTS
  Rbr reg@ drop  \ clear interrupt, receiver data ready
  Lsr reg@ drop  \ clear interrupt, receiver errors
  Msr reg@ drop  \ clear interrupt, modem status changes
  Iir reg@ drop  \ clear THRE interrupt
  004 Ier reg!   \ overrun error, parity error, framing error,
  ;              \ break interrupt



\ character i/o .. JL 5/12/89
\

hex

: Read-Status   ( -- n )
  Lsr reg@  ;             \ read line status

: Read-Character   ( -- c )
  Begin                   \ wait for data
    read-status 01 and
  Until
  Rbr reg@  ;             \ read date

: Send-Character   ( c -- )
  Begin                   \ wait for THRE
    read-status 020 and
  Until
  Thr reg!  ;             \ send data

\ For testing
\ : Echo   ( -- )
\   Begin
\     read-character  dup send-character
\     0d -  If  Else  0a send-character  Endif    \ also send LF when rx CR
\   Again  ;



\ Init-Hardware .. JL 5/12/89
\

: Init-Hardware  ( -- )
  010 03 g!   \ turn interrupts off
  07fff for  noop noop noop  next       \ let hardware settle
  reset-uart  ;


only forth definitions
