((
        TARGEND.FTH

        Author: J Lee, S Pelc

        Date started:   02/88
        Last modified:
                S Pelc, J Lee   10/90
                Added RTX paged support.

        RTXUF target message passer.


        Copyright (c) 1988, 1990
        MicroProcessor Engineering
        133 Hill Lane
        Shirley
        Southampton SO1 5AF
        tel: 0703 631441        (inside UK)
             (+44) 703 631441   (international)
))




\ constants
\

decimal
250 constant max-depth		\ 0 and MAX-DEPTH define stack limits




\ Send & Receive *** TARGET DEPENDENT ***          Jon 09/06/89
\

((
If suitable primitives exist use ALIAS to use the suitable primitives
as WAIT-BYTE and SEND-BYTE. Otherwise define WAIT-BYTE and SEND-BYTE
in a similar fashion to KEY and EMIT respectively.
))

((
\ aliased versions
Alias (Key) Wait-Byte
Alias (Emit) Send-Byte
))


\ non-aliased versions
: Wait-byte	\ -- b
  read-character  ;			\ From serial port to PC host

: Send-byte	\ b --
  send-character  ;			\ To serial port to PC host




\ wait-ack send&ack                                sfp 12/04/89
\

: wait-ack	\ -- ; wait for response from host
  wait-byte ACK! <>			\ get response and check
  IF  Kick-Off  ENDIF  ;		\   Error!, go restart

: send&ack	\ n -- ; send character, wait for acknowldge
  send-byte				\ send byte
  wait-ack  ;				\ get response and check




\ Send-word  receive-word
\

Hex

: Send-word	\ n --
  0 0100 um/mod				\ -- bl bh ; -- rem quot
  send&ack  send&ack  ;			\ send both bytes

: Receive-word	\ -- n
  wait-byte  ACK! send-byte			
  wait-byte  ACK! send-byte
  swap 0100 * +  ;			\ Convert to a word -- n

decimal




\ send-stack receive-stack
\

: send-stack	\ -- ; send stack to host
  depth 0 max				\ get depth, clip underflow
  dup send-word  ?dup			\ tell host what's coming
  if  1-				\ massage for NEXT
      for  r@ pick send-word  next	\ copy it - bottom first
  endif  ;

: receive-stack	\ -- sys ; receive stack from host
  0 sp!					\ clear stack
  receive-word ?dup			\ get depth
  if  1-
      for  receive-word  next		\ get stack - bottom first
  endif  ;




\ Send-message .. J L 24/10/90
\

: Send-message  \ [sys] mess# --
  dup send&ack		\ send command
  CASE				\ request use of host facilities
    push-cmnd>h OF  send-word     ENDOF
    emit-cmnd>h OF  send-word     ENDOF
    key?-cmnd>h OF  receive-word  ENDOF
    key-cmnd>h  OF  receive-word  ENDOF
       .-cmnd>h OF  send-word     ENDOF
    s>t-cmnd>h  OF  receive-stack ENDOF
    s>h-cmnd>h  OF  send-stack    ENDOF
                kick-off
  ENDCASE  ;



\ copy-h>t                                         Jon 07/12/89
\

: copy-h>t      \ addr page len --
  ?dup
  if  1-
      for  2dup				\ -- addr page addr page
	   wait-byte  ACK! send-byte	\ get byte to store
           rot rot c!l			\ store it
           swap 1+ swap
      next
      drop drop
  else  drop drop
  endif
  wait-ack  ;




\ message-passer
\

decimal

there equ exec-point			\ will store code here
  4 allot-ram
there equ save-dpr
  2 allot-ram
there equ save-md
  2 allot-ram
there equ save-sr
  2 allot-ram


: save-regs	\ -- ; preserve APP RTX registers
  dpr@  RAM-PAGE dpr!  seldpr           \ write to monitor RAM-PAGE
  save-dpr !                            \ preserve app DPR
  md@ save-md !                         \ preserve app MD
  sr@ save-sr !                         \ preserve app SR
  selcpr  ;                             \ restore DPR


: restore-regs	\ -- ; restore APP RTX registers
  RAM-PAGE dpr!  seldpr                 \ write to monitor RAM-PAGE
  save-md @ md!                         \ restore MD
  save-sr @ sr!                         \ resore SR
  save-dpr @ dpr!                       \ restore DPR (must be last)
  selcpr  ;
  

hex

: Message-passer
  BEGIN
    wait-byte  ACK! send-byte           \ get and ack. command
    CASE                                \ test command byte
      execute-cmnd>t			\ execute function
      OF  receive-stack wait-ack
		 restore-regs				\ restore user environment
		 seldpr cpr! execute  selcpr	\ long call
		 save-regs				\ save user environment
          s>h-cmnd>h send-message
      ENDOF
      push-cmnd>t			\ push byte to target stack
      OF  receive-word wait-ack			ENDOF
      pop-cmnd>t			\ pop byte from target stack
      OF  send-word wait-ack				ENDOF
      s>t-cmnd>t			\ transfer host stack to target
      OF  receive-stack  wait-ack		ENDOF
      s>h-cmnd>t			\ transfer target stack to host
      OF  send-stack  wait-ack			ENDOF
      @-cmnd>t				\ fetch word from target
      OF  receive-word receive-word	\ get offset and page
	  @l send-word  wait-ack
      ENDOF
      !-cmnd>t				\ store word in target
      OF  receive-word			\ item to store
	  receive-word receive-word	\ address and page
	  wait-ack !l
      ENDOF
      C@-cmnd>t				\ fetch byte from target
      OF  receive-word receive-word	\ address and page
	  C@l send-word  wait-ack
      ENDOF
      C!-cmnd>t				\ store byte in target
      OF  receive-word			\ item to store
	  receive-word receive-word	\ address and page
	  wait-ack C!l
      ENDOF
      copy-cmnd>t
      OF  receive-word receive-word receive-word	\ -- addr page len
	  wait-ack  copy-h>t
      ENDOF
      vers-cmnd>t
      Of  wait-ack  version$ cr count type cr  ENDOF
      execpr-cmnd>t			\ execute opcode
      OF  receive-stack wait-ack
		 seldpr  exec-point !  0A020 exec-point 2+ !  selcpr
		 restore-regs
		 seldpr  RAM-PAGE cpr! [ exec-point >>1 , \\ ]  selcpr
		 save-regs
          s>h-cmnd>h send-message
      ENDOF
          kick-off
    ENDCASE
    depth max-depth u>			\ if stack underflow
    if  0 sp!  SUFL!			\ reset stack and tell host
    else  OK!				\ else say OK
    endif
    send-byte
  AGAIN  ;

\ Kick-Off                                         sfp 19/04/90
\

decimal

: Kick-Off	\ -- ; run at cold start
  0 spr!				\ initial stack pointers
  init-hardware				\ set up serial line
  (init)				\ initialise Forth
\  Begin  rxa?  While  read-character drop  Repeat	\ clear serial comms.
  RAM-PAGE save-dpr !			\ initialise saved RTX state
  message-passer  ;			\ run message passer

\ ' kick-off >>1 0 !
make-turnkey kick-off

