\ MPE RTX-2000 PowerBoard I/O handlers               sfp 11/06/88

((
Copyright (c) 1988-2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeltd.demon.co.uk
     tech-support@mpeltd.demon.co.uk
web: www.mpeltd.demon.co.uk

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
19880516 SFP001 Original release
))

Only Forth Definitions

\ base addresses

also I/O definitions

hex

0FFF8 equ cs8254           \ 8254 timer
0FFF0 equ cs8530           \ 8530 2-channel serial chip
0FFE8 equ cs8255           \ 8255 parallel i/o
0FFE0 equ cs5380           \ 5380 SCSI chip
0FFD8 equ csdack           \ 5380 DACK pin
0FFD0 equ cseop            \ 5380 EOP pin

\ 8254 timer                                       sfp 28/07/88
hex

cs8254 7 + equ control-8254
cs8254 5 + equ ch2-8254
cs8254 3 + equ ch1-8254
cs8254 1 + equ ch0-8254

: settle-8254
  08 for next  ;                        \ settling time

\ baud-base baud                                   sfp 01/04/89
hex

only forth also i/o also Forth definitions

: baud          \ n -- ; give baud/100 e.g. 96 for 9600 baud
  cpu-clock 2@  010 mu/mod  rot drop    \ CPU clock/16
  rot um/mod nip                        \ cpu-clock/(16*baud)
  036 control-8254 c!  settle-8254      \ select timer channel
  dup ch0-8254 c!                       \ output low byte
  >>8 ch0-8254 c!  ;                    \ output high byte

decimal

\ init-8254                                        sfp 01/04/89
hex
only forth also i/o definitions

: init-8254     \ -- ; channels 0,2 as 9600 baud, 1 as 1ms
  baud-rate @ baud                      \ channel 0 at 9600 baud
  076 control-8254 c!  settle-8254      \ channel 1, mode 3
  041 ch1-8254 c!  settle-8254          \
  00 ch1-8254 c!  settle-8254
  0B6 control-8254 c!  settle-8254      \ channel 2, mode 3
  010 ch2-8254 c!  settle-8254          \ /10000 = /02710h
  027 ch2-8254 c!  settle-8254  ;

\ 8255 parallel i/o                                sfp 11/06/88
hex

cs8255 7 + equ control-8255             \ identify ports
cs8255 5 + equ pc-8255
cs8255 3 + equ pb-8255
cs8255 1 + equ pa-8255

: settle-8255   \ -- ; lets 8255 setlle down
  010 for next  ;

: init-8255     \ -- ; Mode 0, PA o/p, PB o/p, PCH i/p, PCL o/p
  08A control-8255 c!  settle-8255
  07 pc-8255 c!  settle-8255  ;

\ 8530 i/o                                         sfp 20/04/90

hex

cs8530 1 + equ cmd-b
cs8530 3 + equ cmd-a
cs8530 5 + equ data-b
cs8530 7 + equ data-a

: settle-8530   \ -- ; settling delay - NEEDED!
  010 for next  ;                       \ settle

\ 82530 SCC interface                              sfp 11/06/88
HEX

: WRAREG                        ( data reg --)
  0F AND cmd-a c!  settle-8530          \ put reg num in wr0
  cmd-a c!  settle-8530  ;              \ put data in reg
: RDAREG                        ( reg -- data)
  0F AND cmd-a c!  settle-8530          \ put reg num in wr0
  cmd-a c@  0FF AND  settle-8530   ;    \ get data, & mask it

: WRBREG                        ( data reg --)
  0F AND  cmd-b c!  settle-8530         \ put reg num in wr0
  cmd-b c!  settle-8530  ;              \ put data in reg
: RDBREG                        ( reg -- data)
  0F AND  cmd-b c!  settle-8530         \ put reg num in wr0
  cmd-b c@  0FF AND  settle-8530   ;    \ get data, & mask it

\ 82530 SCC interface                              sfp 20/04/90
HEX

: init-8530-a
        0C6 09 WRAREG           \ hardware reset
        04E 04 WRAREG           \ *16 8s  2 stop bit
        0C1 03 WRAREG           \ 8 bits per char en rx
        0EA 05 WRAREG           \ 8 bits per char en tx
        000 06 WRAREG
        000 07 WRAREG
        000 0A WRAREG
        005 0B WRAREG           \ set clocks
        006 0C WRAREG           \ low time constant
        000 0D WRAREG           \ top time constant 19200 Baud
        0A0 0E WRAREG ;         \ clock sources

\ 82530 SCC interface                              sfp 20/04/90
HEX

: init-8530-b
        \ hardware reset done by INIT-82530-A
        04E 04 wrbreg           \ *16 8s  2 stop bit
        0C1 03 wrbreg           \ 8 bits per char en rx
        0EA 05 wrbreg           \ 8 bits per char en tx
        000 06 wrbreg
        000 07 wrbreg
        000 0A wrbreg
        005 0B wrbreg           \ set clocks
        006 0C wrbreg           \ low time constant
        000 0D wrbreg           \ top time constant 19200 Baud
        0A0 0E wrbreg ;         \ clock sources

\ 82530 SCC interface                              sfp 11/06/88
HEX
: TXA?          \ -- t/f ; true when ready for another character
  cmd-a c@ 4 AND  ;
: RXA?          \ -- t/f ; true when character available
  cmd-a c@ 1 AND  ;
: txchara       \ char -- ; sends character - N.B. not wait
  data-a c!  ;
: TXA           \ char -- ; sends character, waits first
  begin  TXA?  until
  txchara  ;
: rxchara       \ -- char ; reads last character received
  data-a c@ 0FF AND  ;
: RXA           \ -- char ; waits for next character
  begin  RXA?  until
  rxchara  ;

\ 82530 SCC interface                              sfp 11/06/88
HEX

: TXB?          \ -- t/f
  cmd-b c@ 4 AND  ;

: RXB?          \ -- t/f
  cmd-b c@ 1 AND  ;

: TXB           \ char --
  begin  TXB?  until
  data-b c!  ;

: RXB           \ -- c
  begin  RXB?  until
  data-b c@ 0FF AND  ;

\ Init-Hardware                                     \  20/11/90
hex
only forth also i/o also forth definitions

: init-hardware
  07FFF for  noop noop noop  next       \ let hardware settle
  init-8254  init-8255                  \ initialise hardware
  init-8530-a  init-8530-b
  0EFFF for  r@ c@  r@ c!  next         \ copy ROM into RAM
  00010 3 g!  noop                      \ turn ROM off
;

\ serial handler - buffer layer                    sfp 01/09/90

only forth also i/o definitions

variable ^ip                            \ queue variables
variable ^op
variable buff-count

0100 Equ Rxa-Buff-Size                  \ receiver buffer size
                                        \ must be power of two

create RxA-Buff                         \ keyboard int. buffer
  RxA-Buff-Size Allot

RxA-Buff-Size 1 - constant buff-mask    \ 0100 -> 0FF etc

\ serial handler - buffer layer                    sfp 18/05/88

: ?more-A       \ -- ; samples input and places ip in buffer
  rxa?                                  \ if more in
  if  rxchara                           \ get character
      buff-count @ RxA-Buff-Size <      \ if room
      if  RxA-Buff ^ip @ + c!           \ store character
          ^ip @ 1 + buff-mask and ^ip ! \ update pointer
          buff-count Incr               \ increment count
      else  Drop                        \ or chuck character
      endif
  endif  ;

\ serial handler - buffer layer                    Jon 09/12/88

: (key-a?)      \ -- n ; checks for input
  ?more-A  buff-count @  ;

: (key-a)       \ -- char
  begin
    (key-a?) 0=
  while
    [ also forth definitions ]
    pause
    [ previous definitions ]
  repeat
  RxA-buff ^op @ + c@                   \ get character
  ^op @ 1 + Buff-Mask And ^op !         \ update pointer
  Buff-Count Decr  ;                    \ count down

: (emit-a)      \ char --
  begin  ?more-A  txa? until            \ wait till ready
  txchara                               \ transmit
  Out Incr  ;                           \ count it out

\ (type) (cr)                                      sfp 20/04/90

Decimal

only forth also i/o also Forth definitions

: (Type)        \ addr len -- ; in terms of EMIT
  bounds                                \
  ?Do  I C@ Emit  Loop  ;               \

: (CR)          \ -- ; in terms of EMIT
  13 emit  10 emit  Out Off  ;          \

\ emit type cr key? key                            sfp 20/04/90

Defer Emit      Assign (Emit-A) To-Do Emit
Defer Type      Assign (Type)   To-Do Type
Defer CR        Assign (CR)     To-Do CR
Defer Key?      Assign (Key-A?) To-Do Key?
Defer Key       Assign (Key-A)  To-Do Key

((
Defer Emit      Assign Txa      To-Do Emit
Defer Type      Assign (Type)   To-Do Type
Defer CR        Assign (CR)     To-Do CR
Defer Key?      Assign Rxa?     To-Do Key?
Defer Key       Assign Rxa      To-Do Key
))

\ Model definitions - RTX 2000 PowerBoard          sfp 01/09/90
\ Define serial i/o words in any suitable way

defer ser-emit   assign (emit-a)  to-do ser-emit
defer ser-key    assign (key-a)   to-do ser-key
defer ser-key?   assign (key-a?)  to-do ser-key?

only forth definitions

