\ MPE RTX-2000 4 channel comms card I/O handlers - interrupt driven

pto

Copyright (C) 1988, 1994 MicroProcessor Engineering

Released for use with the MPE Forth Cross Compiler by:

MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
England

tel: +44 703 631441
fax: +44 703 339691
net: mpe@cix.compulink.co.uk

These drivers are set for the two 85C30s on the RTX-2001 4 channel
PC comms card. The 8530s are provided with a 3.6864Mhz crystal.
The default baud rate on all channels is 19200 baud.

\ 8530 i/o

Only Forth Definitions  also I/O definitions

hex

0FFF8 equ cs8530b          \ 8530 2-channel serial chip
0FFF0 equ cs8530a          \ 8530 2-channel serial chip

cs8530a 1 + equ cmd-b
cs8530a 3 + equ cmd-a
cs8530a 5 + equ data-b
cs8530a 7 + equ data-a

cs8530a 2 + constant base-a
cs8530a     constant base-b
cs8530b 2 + constant base-c
cs8530b     constant base-d

: settle-8530   \ -- ; settling delay - NEEDED!
  010 for next  ;                       \ settle

: 8530-init	\ table 8530addr -- ; specify table and base address
  1+ 					\ step to command reg
  swap dup @				\ get count ; -- 8530addr table count
  swap 2+ swap				\ step over count
  1- for
    2dup c@ 0F and swap c!  settle-8530  1+	\ write register#
    2dup c@ swap c!  settle-8530	 1+	\ write register data
  next
  2drop  ;

: wrreg		\ data reg base --
  1+ >r
  0F AND r@ c!  settle-8530		\ put reg num in wr0
  r> c!  settle-8530  ;			\ put data in reg

: rdreg		\ reg base -- data
  1+ >r
  0F and r@ c!  settle-8530		\ put reg num in wr0
  r> c@  0FF and  settle-8530   ;	\ get data, & mask it

: baud          \ n base -- ; give baud e.g. 9600 baud
  >r
  #3,686400 #32 mu/mod  rot drop	\ clock/32
  rot um/mod nip  2-			\ clock/(32*baud) - 2
  0 0100 um/mod				\ split into low and high
  $0D r@ wrreg  $0C r> wrreg  ;

\ initialisation tables

create table-8530a
  #12 ,				\ counter
  09 c,  0C0 c,			\ h/w reset
  04 c,  044 c,			\ *16 8,N,1
  01 c,  000 c,			\ no interrupts
  03 c,  0C0 c,			\ 8 bits per char
  05 c,  060 c,			\ 8 bits per char
  0B c,  0D4 c,			\ set clocks - XTAL, brg, brg, TX=o, XTAL
  0C c,  004 c,			\ low time constant
  0D c,  000 c,			\ top time constant 19200 Baud @ 3.6864MHz
  0A c,  000 c,			\ NRZ transmission
  0E c,  081 c,			\ clock sources, enable BRG
  03 c,  0C1 c,			\ 8 bits per char, en rx
  05 c,  0EA c,			\ 8 bits per char, en tx, RTS DTR

create table-8530b
  #12 ,				\ counter
  09 c,  040 c,			\ channel B reset
  04 c,  044 c,			\ *16 8s  1 stop bit
  01 c,  000 c,			\ no interrupts
  03 c,  0C0 c,           	\ 8 bits per char en rx
  05 c,  060 c,			\ 8 bits per char en tx
  0B c,  050 c,			\ set clocks - no XTAL, brg, brg, TX=i, xx
  0C c,  004 c,			\ low time constant
  0D c,  000 c,			\ top time constant 19200 Baud @ 3.6864MHz
  0A c,  000 c,			\ NRZ transmission
  0E c,  081 c,			\ clock sources, enable BRG
  03 c,  0C1 c,			\ 8 bits per char en rx
  05 c,  0EA c,			\ 8 bits per char en tx, RTS DTR Tx enabled

create table-8530c
  #12 ,				\ counter
  09 c,  0C0 c,			\ h/w reset
  04 c,  044 c,			\ *16 8s  1 stop bit
  01 c,  000 c,			\ no interrupts
  03 c,  0C0 c,           	\ 8 bits per char en rx
  05 c,  060 c,			\ 8 bits per char en tx
  0B c,  050 c,			\ set clocks - no XTAL, brg, brg, TX=i, xx
  0C c,  004 c,			\ low time constant
  0D c,  000 c,			\ top time constant 19200 Baud @ 3.6864MHz
  0A c,  000 c,			\ NRZ transmission
  0E c,  081 c,			\ clock sources, enable BRG
  03 c,  0C1 c,			\ 8 bits per char en rx
  05 c,  0EA c,			\ 8 bits per char en tx, RTS DTR Tx enabled

create table-8530d
  #12 ,				\ counter
  09 c,  040 c,			\ channel B reset
  04 c,  044 c,			\ *16 8s  1 stop bit
  01 c,  000 c,			\ no interrupts
  03 c,  0C0 c,           	\ 8 bits per char en rx
  05 c,  060 c,			\ 8 bits per char en tx
  0B c,  050 c,			\ set clocks - no XTAL, brg, brg, TX=i, xx
  0C c,  004 c,			\ low time constant
  0D c,  000 c,			\ top time constant 19200 Baud @ 3.6864MHz
  0A c,  000 c,			\ NRZ transmission
  0E c,  081 c,			\ clock sources, enable BRG
  03 c,  0C1 c,			\ 8 bits per char en rx
  05 c,  0EA c,			\ 8 bits per char en tx, RTS DTR Tx enabled

\ 8530 SCC interface

HEX

: tx?		\ base -- t/f ; true when ready for another character
  1+ c@  4 and  ;

: rx?		\ base -- t/f ; true when character available
  1+ c@  1 and  ;

: txchar	\ char base -- ; sends character - N.B. not wait
  5 + c!  ;

: rxchar	\ base -- char ; reads last character received
  5 + c@  0FF and  ;

#tasks 1 > if(
: tx		\ char base --
  begin  dup tx? 0=
    while  pause
  repeat
  txchar  ;

: rx		\ base -- char ; waits for next character
  begin  dup rx? 0=
    while  pause
  repeat
  rxchar  ;
)else(
: tx		\ char base --
  begin  dup tx?  until
  txchar  ;

: rx		\ base -- char ; waits for next character
  begin  dup rx?  until
  rxchar  ;
)endif


: txa?		\ -- t/f ; true when ready for another character
  base-a tx?  ;

: rxa?		\ -- t/f ; true when character available
  base-a rx?  ;

: txa		\ char -- ; sends character, waits first
  base-a tx  ;

: rxa		\ -- char ; waits for next character
  base-a rxchar  ;


: txb?		\ -- t/f ; true when ready for another character
  base-b tx?  ;

: rxb?		\ -- t/f ; true when character available
  base-b rx?  ;

: txb		\ char -- ; sends character, waits first
  base-b tx  ;

: rxb		\ -- char ; waits for next character
  base-b rxchar  ;


: txc?		\ -- t/f ; true when ready for another character
  base-c tx?  ;

: rxc?		\ -- t/f ; true when character available
  base-c rx?  ;

: txc		\ char -- ; sends character, waits first
  base-c tx  ;

: rxc		\ -- char ; waits for next character
  base-c rxchar  ;


: txd?		\ -- t/f ; true when ready for another character
  base-d tx?  ;

: rxd?		\ -- t/f ; true when character available
  base-d rx?  ;

: txd		\ char -- ; sends character, waits first
  base-d tx  ;

: rxd		\ -- char ; waits for next character
  base-d rxchar  ;

\ interrupt driven buffer layer

only forth also i/o definitions

hex

\ -- n ; offsets into queue data structure
0 equ ip-offset			\ input pointer
2 equ op-offset			\ output pointer
4 equ n-offset			\ # chars in queue
6 equ base-offset		\ pointer to i/o base
8 equ buffer-offset		\ start of queue proper

080 Equ Rx-Buff-Size			\ receiver buffer size
					\ must be power of two
Rx-Buff-Size 1 - constant buff-mask	\ 0100 -> 0FF etc

: ?more		\ addr -- ; samples input and places ip in buffer
  dup >r				\ save buffer address
  base-offset + @			\ get i/o base address
  dup rx?				\ if more in
  if  rxchar				\ get character, resets interrupt
      r@ n-offset + @  rx-buff-size <	\ if room
      if  r@ buffer-offset +		\ base of buffer
          r@ ip-offset + @  +		\ first free slot in buffer
          c!				\ store character
          r@ ip-offset + @  1 +  buff-mask and  r@ ip-offset + ! \ update pointer
          r@ n-offset + @  1 +  r> n-offset + !			 \ increment count
          Exit				\ get out
      endif
      Drop				\ no room, chuck character
      r> drop				\ discard saved buffer address
      Exit				\ get out
  endif
  drop					\ discard i/o address
  r> drop				\ discard saved buffer address
;


: buff-key?	\ buffer-addr -- n ; checks for input
  n-offset + @  ;

: buff-key	\ buffer-addr -- char
  >r
  begin
    di
    r@ n-offset + @ 0=
   while
     ei
     [ also forth definitions ]
     pause
     [ previous definitions ]
  repeat
  r@ buffer-offset +			\ base of buffer
  r@ op-offset + @  +			\ pointer into buffer
  c@					\ get character
  r@ op-offset + @  1 +  Buff-Mask And  r@ op-offset + !    \ update pointer
  r@ n-offset + @  1 -  r> n-offset + !			    \ decrement count
  ei
;
\ buffers and KEY? KEY EMIT functions

create RxA-Buff	\ -- addr ; keyboard interrupt buffer
  0 ,  0 ,  0 ,				\ initialised pointers
  base-a ,				\ address of 8530
  rx-buff-size allot			\ buffer space

create RxB-Buff	\ -- addr ; keyboard interrupt buffer
  0 ,  0 ,  0 ,				\ initialised pointers
  base-b ,				\ address of 8530
  rx-buff-size allot			\ buffer space

create RxC-Buff	\ -- addr ; keyboard interrupt buffer
  0 ,  0 ,  0 ,				\ initialised pointers
  base-c ,				\ address of 8530
  rx-buff-size allot			\ buffer space

create RxD-Buff	\ -- addr ; keyboard interrupt buffer
  0 ,  0 ,  0 ,				\ initialised pointers
  base-d ,				\ address of 8530
  rx-buff-size allot			\ buffer space


: key-a?	\ -- t/f
  rxa-buff buff-key?  ;

: key-a		\ -- char
  rxa-buff buff-key  ;

: emit-a	\ char --
  base-a tx  Out Incr  ;


: key-b?	\ -- t/f
  rxb-buff buff-key?  ;

: key-b		\ -- char
  rxb-buff buff-key  ;

: emit-b	\ char --
  base-b tx  Out incr  ;


: key-c?	\ -- t/f
  rxc-buff buff-key?  ;

: key-c		\ -- char
  rxc-buff buff-key  ;

: emit-c	\ char --
  base-c tx  out Incr  ;


: key-d?	\ -- t/f
  rxd-buff buff-key?  ;

: key-d		\ -- char
  rxd-buff buff-key  ;

: emit-d	\ char --
  base-d tx  out incr  ;


: ei3-action	\ -- ; action of EI3 ISR
  rxa-buff ?more
  rxb-buff ?more  ;

: ei5-action	\ -- ; action of EI5 ISR
  rxc-buff ?more
  rxd-buff ?more  ;

: init-int-AB	\ -- ; initialise interrupts for channels A and B
  010 1 base-a wrreg			\ enable receive interrupt
  010 1 base-b wrreg			\ enable receive interrupt
  8 9 base-a wrreg			\ enable master interrupt
  assign ei3-action to-do ei3-isr	\ assign interrupt action
  ei3 unmask				\ enable EI5
  ei					\ global interrupt enable
;

: init-int-CD	\ -- ; initialise interrupts for channels C and D
  010 1 base-c wrreg			\ enable receive interrupt
  010 1 base-d wrreg			\ enable receive interrupt
  8 9 base-d wrreg			\ enable master interrupt
  assign ei5-action to-do ei5-isr	\ assign interrupt action
  ei5 unmask				\ enable EI5
  ei					\ global interrupt enable
;

\ 8255 parallel i/o                                sfp 11/06/88

hex

0FFE8 equ cs8255           \ 8255 parallel i/o

cs8255 7 + equ control-8255             \ identify ports
cs8255 5 + equ pc-8255
cs8255 3 + equ pb-8255
cs8255 1 + equ pa-8255

: settle-8255   \ -- ; lets 8255 setlle down
  010 for next  ;

: init-8255     \ -- ; Mode 0, PA o/p, PB o/p, PCH i/p, PCL o/p
		\ N.B. reset line is connected to 8530a DTR line
  06A 5 base-a wrreg  0EA 5 base-a wrreg	\ pulse reset line high, then low
  08A control-8255 c!  settle-8255
  07 pc-8255 c!  settle-8255  ;

\ Init-Hardware                                     \  20/11/90

hex

only forth also i/o also forth definitions

: init-hardware
  07FFF for  noop noop noop  next       \ let hardware settle
  mask-all				\ disable all interrupts
  table-8530a base-a 8530-init		\ initialise channel A
  table-8530b base-b 8530-init		\ initialise channel B
  table-8530c base-c 8530-init		\ initialise channel C
  table-8530d base-d 8530-init		\ initialise channel D
  init-8255				\ must initialise 8530a first
  0EFFF for  r@ c@  r@ c!  next         \ copy ROM into RAM
  00010 3 g!  noop                      \ turn ROM off, disables interrupts!
  init-int-AB init-int-CD		\ init ints for serial channels
;


\ (type) (cr)                                      sfp 20/04/90

Decimal

only forth also i/o also Forth definitions

: (Type)        \ addr len -- ; in terms of EMIT
  bounds
  ?Do  I C@ Emit  Loop  ;

: (CR)          \ -- ; in terms of EMIT
  13 emit  10 emit  Out Off  ;


\ emit type cr key? key
\ for interrupt driven i/o

Defer Key?      Assign Key-d?   To-Do Key?
Defer Key       Assign Key-d    To-Do Key
Defer Emit      Assign Emit-d   To-Do Emit
Defer Type      Assign (Type)   To-Do Type
Defer CR        Assign (CR)     To-Do CR


\ Model definitions - RTX 2000 PowerBoard          sfp 01/09/90
\ Define serial i/o words in any suitable way

defer ser-emit   assign emit-d  to-do ser-emit
defer ser-key    assign key-d   to-do ser-key
defer ser-key?   assign key-d?  to-do ser-key?


only forth definitions

\ miscellaneous i/o

only forth also i/o also Forth definitions

hex

: led-on	\ -- ; turn board LED on
  0EA 5 base-b wrreg  ;

: led-off	\ -- ; turn board LED off
  06A 5 base-b wrreg  ;

only forth definitions

\ all done

decimal

;p
