\ SCSI interface words for RTX-2000                sfp 24/10/88

pto

Stephen Pelc

Copyright (C) 1988
MicroProcessor Engineering
133 Hill Lane
Shirley
Southampton SO1 5AF

tel: 0703 631441
     (+44) 703 631441
))


only forth definitions

\ Interactive load screen                          sfp 27/10/88
((
: ;S
  r> drop  ;

: Equ                                   \ synonym
  Constant  ;

: L:                                    \ Xcompiler synonym
  Create  ;

also  I/O definitions
))
\ variables                                        sfp 25/10/88

variable scsi-drive                     \ current SCSI drive

\ SCSI constants                                   sfp 25/10/88
hex

0FFD0 constant SCSI-base                \ base address
scsi-base 1 +   constant scsi-data      \ data
scsi-base 3 +   constant scsi-init-cmd  \ initiator
scsi-base 5 +   constant scsi-mode      \ mode
scsi-base 7 +   constant scsi-trgt-cmd  \ target
scsi-base 9 +   constant scsi-status    \ status (read)
scsi-base 9 +   constant scsi-select    \ select (write)
scsi-base 0B +  constant scsi-bus       \ Bus/Status (read)
scsi-base 0D +  constant scsi-input     \ latched input
scsi-base 0F +  constant scsi-reset     \ reset interrupts

\ negate-bus assert-bus clear-scsi-int             sfp 25/10/88
hex

: negate-bus    \ -- ; turn output drivers off
  1 scsi-init-cmd reset-bit  ;          \ reset bit 0

: assert-bus    \ -- ; enable output drivers
  1 scsi-init-cmd set-bit  ;            \ set bit 0

: clear-scsi-int        \ --
  scsi-reset c@ drop  ;                 \ read reg to clear

\ reset-scsi                                       sfp 27/10/88
hex

: reset-5380    \ -- ;
  0 scsi-init-cmd c!                    \ clear registers
  0 scsi-data c!                        \
  0 scsi-mode c!                        \
  0 scsi-trgt-cmd c!                    \
  0 scsi-select c!                      \
  clear-scsi-int  ;                     \ reset pending INT

\ reset-scsi                                       sfp 27/10/88
hex

: reset-scsi    \ -- ; reset bus
  reset-5380                            \ reset chip
  080 scsi-init-cmd c!                  \ activate RST
  08000 for next                        \ wait min 25us
  000 scsi-init-cmd c!                  \ release RST
  clear-scsi-int                        \ reset pending INT
  0F000 for noop noop noop next  ;      \ wait a bit

\ free? busy?                                      sfp 25/10/88
hex
: busy?         \ -- t/f true if bus busy
  040 scsi-status test-bit 0<>  ;       \ BUSY true

: free?         \ -- t/f ; true if bus free
  042 scsi-status test-bit 0=  ;        \ BUSY and SEL false

: wait-free     \ -- ; wait for bus to come free
  begin  free? free? and 0=             \ 2 samples
    while  pause                        \  both free
  repeat  ;                             \ are needed

\ >bus                                             sfp 25/10/88

: >bus          \ b -- ; put byte on bus
  scsi-data c!  ;

: assert-sel    \ --
  4 scsi-init-cmd set-bit  ;
: negate-sel    \ --
  4 scsi-init-cmd reset-bit  ;

\ get-phase phase?                                 sfp 25/10/88
hex

: get-phase     \ -- n ; tests MSG C/D I/O
  scsi-status c@ 01C and  ;

: phase?        \ -- n ; wait for stable phase
  -1                                    \ impossible value
  begin  0100 for next                  \ settle before next
         get-phase tuck =               \ new = old ?
  until  ;                              \

\ phases                                           sfp 25/10/88
hex

000 constant data-out-phase
004 constant data-in-phase
008 constant command-phase
00C constant status-phase

018 constant msg-out-phase
01C constant msg-in-phase

\ selection                                        sfp 25/10/88
hex

: match-phase   \ --
  phase? 2/ 2/ scsi-trgt-cmd c!  ;

: selection     \ ID -- status ; 0=success
  match-phase                           \
  >bus assert-bus assert-sel            \ Put out ID, SEL
  -1  08000 0                           \ try for a while
  do  busy?                             \ if BUSY
      if  drop 0  leave  endif          \ leave with SUCCESS
  loop                                  \
  negate-sel  ;                         \ release SEL

\ .phase                                           sfp 25/10/88
hex

: .phase        \ -- ; show current SCSI bus phase
  case  phase?
    data-out-phase      of ." Data Out"         endof
    data-in-phase       of ." Data In"          endof
    command-phase       of ." Command"          endof
    status-phase        of ." Status"           endof
    msg-out-phase       of ." Message Out"      endof
    msg-in-phase        of ." Message In"       endof
                           ." UNDEFINED"
  endcase
  ."  phase"  ;

\ req? wait-req assert-ack negate-ack              sfp 27/10/88
hex

: req?          \ -- state
  scsi-status c@ 020 and  ;

: wait-req      \ -- ; wait for REQ asserted
  begin  req?  until  ;

: assert-ack    \ --
  scsi-init-cmd c@  010 or  scsi-init-cmd c!  ;
: negate-ack    \ --
  scsi-init-cmd c@  0EF and  scsi-init-cmd c!  ;

\ data-in data-out                                 sfp 27/10/88

: data-in       \ -- b ; read data from SCSI bus
  wait-req                              \ wait for REQ true
  scsi-data c@                          \ read data
  assert-ack                            \ set ACK
  begin  req? 0=  until                 \ wait for REQ false
  negate-ack  ;                         \ release ACK

: data-out      \ b -- ; write data to SCSI bus
  wait-req                              \ wait for REQ asserted
  scsi-data c!                          \ put data on bus
  assert-ack                            \ set ACK
  begin  req? 0=  until                 \ wait for REQ false
  negate-ack  ;                         \ release ACK

\ get-scsi-status get-msg-in                       sfp 25/10/88

: get-scsi-status       \ -- b
  begin  phase? status-phase =  until
  match-phase  data-in   ;              \ read status return

: get-msg-in            \ -- b
  begin  phase? msg-in-phase =  until
  match-phase  data-in  ;

: get-completion        \ -- ; error if command not complete
  get-msg-in
  abort" SCSI command not completed"  ;

\ >scsi scsi>                                      sfp 27/10/88

: >scsi         \ addr len -- ; send bytes to SCSI
  dup 0= if  2drop  exit  endif         \ get out on a cock up
  match-phase  1-                       \ allow for FOR NEXT
  for  dup c@ data-out 1+  next         \ xmit data
  drop  ;                               \ drop address

: scsi>         \ addr len - ; read bytes from SCSI
  dup 0= if  2drop  exit  endif         \ get out on a cock up
  wait-req  match-phase  1-             \ allow for FOR NEXT
  for  data-in over c!  1+  next        \ receive data
  drop  ;                               \ drop address

\ send-data-out recv-data-in                       sfp 27/10/88

: send-data-out         \ addr len --
  begin phase? data-out-phase = until   \ wait for data out req
  >scsi  ;                              \ send data

: recv-data-in          \ addr len --
  begin phase? data-in-phase = until    \ wait for data in req
  scsi>  ;                              \ read data

\ finish-command                                   sfp 28/10/88

create drive-table  \ -- addr ; holds drive selects
  001 c,  002 c,  004 c,  008 c,        \ first 4 drives
  010 c,  020 c,  040 c,  080 c,        \ second four drives

create drive-sizes  \ -- addr ; holds drive sizes
  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,  0 ,

: drive-size    \ -- n
  drive-sizes scsi-drive @ 2* + @  ;

\ finish-command start-command                     sfp 27/10/88

: finish-command        \ -- ; wait for command phase to finish
  begin  phase? command-phase <>  until \ let it finish
  010 for next  ;                       \ let it settle

: start-command         \ -- status ; start command process
  wait-free
  drive-table  scsi-drive @ +  c@       \ fetch id-code
  selection                             \ choose drive
  match-phase  assert-bus   ;           \ set up chip

\ issue-command6                                   sfp 27/10/88

: issue-command6        \ addr --
  wait-req
  6 >scsi  ;                            \ six byte commands

\ get-data-in                                      sfp 27/10/88

create sense-buffer     \ -- addr
  020 allot                             \ 32 bytes

: get-sense-data        \ -- ; part of request-sense
  wait-req match-phase
  sense-buffer 8 scsi>                  \ read first 8 bytes
  sense-buffer 7 + c@ ?dup              \ read remaining
  if  sense-buffer 8 + swap scsi>
  endif  ;

\ sense-buffer request-sense                       sfp 28/10/88
hex
create ct-3         \ -- addr
  3 c,  0 c,  0 c,  0 c,  01B c,  0 c,

: request-sense \ -- status
  start-command if -1 exit endif        \ prepare
  ct-3 issue-command6                   \ o/p command
  negate-bus  finish-command            \ finish command
  wait-req  phase? data-in-phase =      \ read data if any
  if  get-sense-data   endif
  wait-req  phase? status-phase =       \ read status if a
  if  get-scsi-status  else  0  endif
  wait-req  phase? msg-in-phase =       \ read message if any
  if  get-completion  endif  ;

\ ?sense-error                                     sfp 27/10/88
hex
: ?sense-error  \ n -- n ; issue request-sense if non-zero
  dup                                   \ error ?
  if  reset-scsi                        \ reset bus
      request-sense drop                \ read sense-buffer
  endif  ;
((
: ?sense-error  \ n -- n ; issue request-sense if non-zero
  dup                                   \ error ?
  if  request-sense drop                \
      sense-buffer here 020 cmove
      reset-scsi                        \ reset bus
      request-sense drop                \ read sense-buffer
      here sense-buffer 020 cmove
  endif  ;
))
\ test-unit-ready                                  sfp 28/10/88

create ct-0                 \ -- addr
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

: test-unit-ready       \ -- status
  start-command if -1 exit endif        \ prepare
  ct-0 issue-command6                   \ 6 bytes of 0
  negate-bus finish-command             \
  wait-req get-scsi-status  ;

\ read-capacity                                    sfp 27/10/88
hex

: read-capacity \ -- capacity status
  start-command if 0 -1 exit endif      \ prepare
  wait-req match-phase
  025 data-out  8 for 0 data-out next   \ o/p command
  finish-command  negate-bus            \ finish command
  sense-buffer 8 recv-data-in           \ read response
  sense-buffer 2+ @                     \ retrieve #blocks
  get-scsi-status                       \ return status
  get-completion  ?sense-error  ;       \ disconnect

\ format-unit                                      sfp 28/10/88
hex
create ct-4         \ -- addr ; command table
  04 c, 010 c,  0 c,  0 c,  3 c,  0 c,
create dlh          \ -- addr ; defect list header
  0 c,  080 c,  0 c,  0 c,
: format-unit   \ -- status
  start-command if -1 exit endif        \ prepare
  ct-4 issue-command6                   \ o/p command
  finish-command                        \ finish command
  wait-req  dlh 4 send-data-out         \ send defect list
  negate-bus                            \ turn bus off
  wait-req                              \ wait request
  get-scsi-status                       \ return status
  get-completion  ?sense-error  ;       \ disconnect

\ mode-buffer mode-sense                           sfp 28/10/88
hex
create mode-buffer      \ -- addr ; buffer space
  010 allot

create ct-01A               \ -- addr ; command table
  01A c,  0 c,  0 c,  0 c,  0C c,  0 c,

: mode-sense            \ -- status
  start-command if -1 exit endif        \ prepare
  ct-01A issue-command6                 \ o/p command
  negate-bus  finish-command            \ clean up
  mode-buffer 0C recv-data-in           \ read returned data
  wait-req                              \
  get-scsi-status  get-completion  ;    \ tidy up

\ mode-select                                      sfp 28/10/88
hex
create ct-015       \ -- addr ; command table
  015 c,  0 c,  0 c,  0 c,  0C c,  0 c,

: mode-select   \ -- status
  mode-buffer 0C erase                  \ clean output buffer
  8 mode-buffer 3 + c!                  \ block descriptor len
  0400 mode-buffer 0A + !               \ block length
  start-command if -1 exit endif        \ prepare
  ct-015 issue-command6                 \ o/p command
  finish-command                        \ clean up
  mode-buffer 0C send-data-out          \ xmit data
  negate-bus  wait-req                  \
  get-scsi-status  get-completion  ;    \ tidy up

\ write-m-scsi write-scsi                          sfp 27/10/88
hex
: write-m-scsi  \ addr blk n -- status
  start-command                         \ start it
  if  2drop drop  -1 exit  endif        \ check ok
  0A data-out  0 data-out               \ opcode, block v.hi
  swap dup >>8 data-out  data-out       \ block number
  dup data-out  0 data-out              \ #blocks, 0
  finish-command  wait-req              \
  0400 um* drop send-data-out           \ copy data
  negate-bus  wait-req                  \
  get-scsi-status  get-completion  ?sense-error  ;

: write-scsi    \ addr blk -- status
  1 write-m-scsi  ;

\ read-m-scsi read-scsi                            sfp 27/10/88
hex
: read-m-scsi   \ addr blk n -- status
  start-command                         \ start it
  if  2drop drop  -1 exit  endif        \ check ok
  08 data-out  0 data-out               \ opcode, block v.hi
  swap dup >>8 data-out  data-out       \ block number
  dup data-out  0 data-out              \ #blocks, 0
  finish-command  negate-bus            \
  wait-req  phase? data-in-phase =      \ check data-in or stat.
  if  0400 um* drop recv-data-in        \ copy data
  else  2drop                           \ discard addr & len
  endif                                 \
  wait-req                              \
  get-scsi-status  get-completion       \ get status & complete
  ?sense-error  ;

\ read-scsi                                        sfp 27/10/88
hex

: read-scsi     \ addr blk -- status
  1 read-m-scsi  ;

\ init-scsi                                        sfp 27/10/88

: init-scsi     \ -- ; set everything up
  reset-scsi
  request-sense abort" Can't reset SCSI"
  read-capacity abort" Can't read capacity"
  drive-sizes scsi-drive @ 2* + !  ;

\ test-drive                                       sfp 27/10/88
hex
: test-drive    \ --
  cr ." Blocks on disc = " decimal drive-size u. ." (decimal)"
  0
  begin  dup 03FF and 0=
         if  cr  dup 0A >>n 3 .r  2 spaces  endif
         dup 01F and 0=
         if  ." ."  endif
         here over read-scsi         \ see if error
         if  dup u.  endif           \ show block#
         1+  dup drive-size 1+ =  halt? dup
         if  cr ." Test aborted"  endif
         or
  until
  drop  cr  ;
