\ Interrupt handlers for all RTX systems

pto

	********************************************
	***                                      ***
	*** INTERUPT.FTH                         ***
	***                                      ***
	*** Description:                         ***
	***   Interrupt ISRs and control words	 ***
	***   for RTX2000/1/10 ROM Target. 	 ***
	***                                      ***
	*** History:                             ***
	***   V1.000 J Lee 12/8/93               ***
	***   Original release.                  ***
	***                                      ***
	********************************************

\ handlers

only forth definitions
decimal


((
****************************************************
****						****
**** 		Interrupt handlers		****
****						****
****************************************************
))

: Default-Interrupt		( -- ; default ISR action )
;

\
\ These DEFERed words are executed on interrupt.  Assign
\ the interrupt code to the appropriate xxxx-isr words.
\
defer swi-isr
defer ei5-isr
defer ei4-isr
defer ei3-isr
defer timer2-isr
defer timer1-isr
defer timer0-isr
defer ei2-isr
defer rsv-isr
defer psv-isr
defer rsu-isr
defer psu-isr
defer ei1-isr
defer nmi-isr
defer phantom-isr

\ Default handlers. Assign doesn't work on Umbilical (not initialised)
\ so if you want to initialise a handler put the assign in your
\ application startup word.

((
assign default-interrupt to-do swi-isr
assign default-interrupt to-do ei5-isr
assign default-interrupt to-do ei4-isr
assign default-interrupt to-do ei3-isr
assign default-interrupt to-do timer2-isr
assign default-interrupt to-do timer1-isr
assign default-interrupt to-do timer0-isr
assign default-interrupt to-do ei2-isr
assign default-interrupt to-do rsv-isr
assign default-interrupt to-do psv-isr
assign default-interrupt to-do rsu-isr
assign default-interrupt to-do psu-isr
assign default-interrupt to-do ei1-isr
assign default-interrupt to-do nmi-isr
assign default-interrupt to-do phantom-isr
))

((
****************************************************
****						****
**** 	Interrupt SAVE & RESTORE		****
****	  - See VECTORS.FTH for use.		****
****						****
****************************************************
))

\ assume interrupts are enabled to have called this word
: Save-int		( -- sr md cr ; save processor state )
  sr@ md@ cr@			\ preserve registers
  dup $000B and  $10 or  cr!	\ Motorola mode, keep interrupts disabled
  selcpr			\ set code page for data access
;

: Restore-int		( sr md cr -- ; restore processor state )
  cr! md! sr!			\ restore processor's registers
;


((
****************************************************
****						****
**** 	 	Interrupt control words		****
****						****
****************************************************
))


$0080 constant Timer0		\ IMR mask setting
$0100 constant Timer1		\ IMR mask setting
$0200 constant Timer2		\ IMR mask setting
$0002 constant EI1		\ IMR mask setting
$0040 constant EI2		\ IMR mask setting
$0400 constant EI3		\ IMR mask setting
$0800 constant EI4		\ IMR mask setting
$1000 constant EI5		\ IMR mask setting
$2000 constant swi		\ IMR mask setting
$0020 constant rsv		\ IMR mask setting
$0010 constant psv		\ IMR mask setting
$0008 constant rsu		\ IMR mask setting
$0004 constant psu		\ IMR mask setting


: Di			( -- ; disable all maskable interrupts )
  cr@  $10 or  cr!
;

: Ei  			( -- ; enable all unmasked interrupts )
  cr@  cr!			\ Int. control bit. Always reads as 0 (enable)
;

: Mask-All		( -- ; mask all interrupts )
  $3FFE imr!
;

: Mask			( mask -- ; use mask to set IMR )
  imr@  or  imr!
;

: UnMask		( mask -- ; use mask to unmask IMR )
  not  imr@  and  imr!
;

: Masked?		( mask -- t/f ; return TRUE if mask flag set )
  imr@  and  0<>
;



