\ RTX RAM PowerForth - Multi-Tasker

pto

MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
England

tel: (+44) 703 631441
fax: (+44) 703 339691

First Release: 5/02/92 Beta test v1.001

Changes:	See RELEASE.DOC for full details


\ Task Control Block (TCB) layout                  sfp 23/11/88

only forth definitions

 + 0,1  - SSP   Saved Parameter and Return Stack Pointer
 + 2    - STAT  BIT 7   1 = running, 0 = halted
                BIT 6   1 = message pending
                BIT 5   1 = event triggered
                BIT 4   1 = event has run
		BIT 3 - 0 = user bits
 + 3    - TASK  Task # that sent message here
 + 4,5  - MESG  Message address
 + 6,7  - ACTN  Task action cfa
 + 8,9  - EVNT  Event action cfa

\ define TCB area offsets

decimal

\ Bit offsets
  16  constant Evrunbit				\ event run bit
  32  constant Evsetbit				\ event set bit
  64  constant Msgbit				\ message bit
  128 constant Runbit				\ task run bit

\ offsets within the TCB save area
  0 constant tcbsp			\ task's SP
  2 constant tcbst			\ task status byte
  3 constant tcbid			\ id of message sender
  4 constant tcbmsg			\ task message data
  6 constant tcbaction			\ CFA of main word
  8 constant tcbevent			\ CFA of event word

  10 constant tcblen			\ length of task control block

Variable Task-temp			\ For random access into RTX stack
Variable Spr-temp			\ For random access into RTX stack
Variable Curtask			\
Variable Task#				\

\ reserve RAM space for variables and TCBS

stack-size #tasks / dup 256 * +		\
  equ stack-gap				\ gap between stacks

create tcbs
  tcblen #tasks * allot			\ reserve TCB space
here constant endtcbs

\ self                                             sfp 23/11/88
hex

: self		\ -- task# ; push current task#
  task# c@  $7F and			\
;

: >tcb		\ task# -- 'tcb ; returns address of task#'s tcb
  tcblen * tcbs +			\
;

\ (pause) .............................      MBN-sfp 23/11/88

: (pause)		\ -- ; the scheduler
  task# c@ Runbit and 0=		\ top bit reset ?
  If					\ Yes, task is active
    user-base @ 0 0			\ flush data stach cache, save user area
    0 >r				\ flush return stack cache
    spr@ 
    [ 2000? ] if( $102 - )endif		\ adjust stack pointer on RTX 2000
    curtask @ tcbsp +  !		\ save SP&RP in TCB
\
\ select next task to run
\
    Begin				\
      user-base dup @ per-task + swap !	\ newuser=olduser+usersize
      task# dup c@ 1+ swap c!		\ increment task#
      curtask dup @ tcblen + swap !	\ nexttcb=currtcb+tcblen
      curtask @ endtcbs =		\ at end of TCBs ?
      If				\ Yes,
        tcbs curtask !			\   reset current tcb
	0 task# c!			\   reset current task#
	init-u0 user-base !		\   reset user base
      Endif				\
      curtask @ tcbst + c@		\ status for task
    Until				\ check for message,event or task running
\
\ run selected task
\
    curtask @ tcbsp + @ spr!		\ restore SP from TCB
    drop drop user-base !		\ restore data
    r> drop				\ restore return
\
\ run event if needed
\
    curtask @ tcbst + c@		\ status byte
    evsetbit and			\ event to be run ?
    If					\ Yes,
      curtask @ tcbst +			\   address of status byte
      dup c@ evrunbit or		\   indicate event has been run
      evsetbit not and swap c!		\   clear event set bit
      curtask @ tcbevent + @ 2* execute	\   run event
    Endif				\

  Endif					\
;	 

\ restart                                          sfp 23/11/88
HEX

: Restart    \ task# -- ; mark task TCB as running
  >tcb tcbst +				\ address of status field for task#
  Runbit swap Set-bit			\ indicate active by setting bit7
;

\ halt                                             sfp 23/11/88
HEX                                     \
                                        \
: Halt       \ task# -- ; reset running bit in TCB
  >tcb tcbst +				\ address of status field for task#
  Runbit swap Reset-bit			\ indicate halt by resetting bit7
;

\ set-event Reset-event Clr-run-event

: set-event	\ task# -- ; set task's event flag
  >tcb tcbst +				\ add of task's status
  evsetbit swap Set-bit			\ set event bit
;

: Reset-event	\ task# -- ; set task's event flag
  >tcb tcbst +				\ add of task's status
  evsetbit swap Reset-bit		\ reset event bit
;

: Clr-run-event	\ task# -- ; clear event run flag
  >tcb tcbst +				\ add of task's status
  evrunbit swap Reset-bit		\ reset event bit
;

\ event? ?event

: event?     \ task# -- flag ; true if task had event
  >tcb tcbst + c@ evrunbit and		\ event set ?
;

: ?event     \ -- ; run event if flag set
  Self event?				\ Event to be run ?
  If					\ Yes,
    Curtask @ tcbevent + @ 2* execute	\   get cfa from tcb and run it
  Endif					\
;

\  to-task to-event                                sfp 23/11/88
HEX                                     \

: to-task	\ cfa task# ---
  tuck >tcb tcbaction + !		\ set into TCB of task
  init-task				\ initialise it
;

: to-event	\ cfa event# ---
  >tcb tcbevent + !			\ set into TCB of task
;

\ (wait)					sfp 12/02/90

: (wait)          \ n -- ; reschedule n times, testing for events
  ?dup					\ 0 = for ever
  if					\
    0 do  pause  loop			\ spin n times
  else					\
    begin       			\ wait for ever
      pause				\ test event, schedule
      self msg?				\ spin if no message
    until				\
  endif					\
;

\ wait-event/msg stop                              sfp 12/02/90

: wait-event/msg        \ -- ; wait for message or event trigger
  begin  self msg?			\ check message to me
         self event?			\ check event to me
         or 0=				\ while neither
    while  pause			\ schedule
  repeat				\ test again
;

: stop          \ -- ; halts current task
  self halt  pause			\
;

\ (send-msg)                                       sfp 12/02/90
hex

: msg?       \ task# -- flag ; true if task has message
  >tcb tcbst + c@ msgbit and		\ Message available ?
;

: (send-msg) \ addr task# -- ; store message in task
  >tcb					\
  tuck tcbmsg + !			\ save message
  Self over tcbid + !			\ save current task# in tcb
  tcbst + msgbit swap set-bit		\ indicate new message
;

: (get-message)      \ -- addr task#
  curtask @ tcbmsg + @			\ get message
  curtask @ tcbid + @			\ get task id
  Msgbit curtask @ tcbst + Reset-bit	\ clear message pending bit
;

\ send-message get-message                         sfp 12/02/90
HEX

: send-message  \ addr task# -- ; send message to task
  Begin					\
    dup msg?				\ if task already has a message
  While					\
    pause				\ wait
  Repeat				\ and try again
  (send-msg)				\ send the message
;

: get-message   \ -- addr task# ; wait for any message
  Begin					\
    self msg? 0=			\ check for a pending message
  While					\
    pause				\ wait for it
  Repeat				\ try again
  (get-message)				\ read it
;

\ single multi                                     sfp 23/11/88
HEX

: multi      \ -- ; enable scheduler by setting flag
  task# Runbit swap Reset-bit		\
;

: single     \ -- ; disable scheduler by clearing flag
  task# Runbit swap Set-bit		\
;

\ init-task                                        sfp 12/02/90

: init-task	\ task# -- ; set up and initialise task
\
\ save current stack state so that we can switch to a new stack
\
  task-temp !				\ save task address
  user-base @  0 0  0 >r		\ flush stack caches and save user area
  spr@ 
  [ 2000? ] if( $102 - )endif		\ adjust stack pointer on RTX 2000
  spr-temp !	   	    		\ save current stack pointer
\
\ switch to the stack for the task to be set up
\
  task-temp @ stack-gap * spr!		\ tasks stack
  task-temp @ >tcb			\ address of tcb
  tcbaction + @ 2* >r			\ stash cfa on return stack
  task-temp @ per-task * init-u0 +	\ start of user area
  0 0  0 >r				\ flush cache
  spr@					\ current stack pointers
  [ 2000? ] if( $102 - )endif 		\ adjust stack pointer on RTX 2000
  task-temp @ >tcb tcbsp + !		\ put SPR into TCB
\
\ restore previous stack state
\
  spr-temp @ spr!			\ restore stack
  r> drop  drop	drop  user-base !	\ refill cache and restore user area

  0 >tcb tcbst c!			\ reset status
;

: activate      \ task# -- ; start task from scratch
  dup init-task				\ reset TCB
  restart				\ mark as running
;

: init-user	\ task# -- ; initialise usr area for task#
  per-task * init-u0 +			\ start of tasks user area
  init-u0 swap per-task cmove		\ copy task0 user area
;

: init-tcbs     \ -- ; reset all TCBs
  0 tcbs tcbst + c!			\ clear status of task 0
  ['] cold tcbs tcbaction + !		\ IP of task 0
  #tasks 1				\ for remaining n tasks
  ?Do					\
     i init-task			\ initialise task
     i init-user			\
  Loop                                  \
  tcbs curtask !			\ Forth task is current
  0 task# c!				\ current task
  0 restart				\ task 0 is Forth
;

\ init-multi                                       sfp 12/02/90
hex

: init-multi    \ -- ; initialisation with multi-tasking
  init-tcbs                             \ initialise TCBs
  assign (pause) to-do pause		\ set pause to be scheduler
  assign (wait) to-do wait		
  multi					\ mark tasker enabled
;
 
