\ index generator for glossaries, builds on MPE PC PowerForth
\
\ Code (c) MicroProcessor Engineering Ltd., 1991
\
\   MicroProcessor Engineering Ltd
\   133 Hill Lane
\   Southampton
\   SO1 5AF
\   Phone: (+44) 703 631441
\   Fax:   (+44) 703 339691
\
\ Supplied for use with PC PowerForth, in conjunction with XShell v3 only
\
\ Author: Paul Gallienne


: app ;

\ constants & variables

2048 constant load-size
20   constant record-size

create buffer      load-size allot
create index-buf   1000      allot

pcb index-pcb
pcb gloss-pcb

2variable file-ptr
2variable end-of-gloss

variable  index-ptr


\ constants

hex
create 'header-code
  0A0D ,  0A0D ,  9D c,
decimal

variable end-of-buf

0  constant false
-1 constant true


\ move-file-ptr

asm

hex

code move-file-ptr	\ offset-from-start handle -- t/f ;
  cx pop  dx pop		\ get double word cx=high, dx=low
  ax, # 4200 mov		\ set to move pointer from current pos.
  21 int			\ move it !
  cy,				\ all ok if carry is reset
  if,   bx, # false mov	\ so return 0
  else, bx, # true  mov	\ or return -1
  endif,			\
  next,			\
end-code			\

decimal

unhook-asm


\ at-end?

: at-end?	\ -- t/f ; at end of glossary
  load-size		\ size of buffer
  end-of-buf @		\ amount read last time
  <>			\ if not the same, at end
;

: d+!		\ d a -- ; as +! but with doubles
  dup 2@		\ get value
  rot >r d+ r>		\ add to addition
  2!			\ store it back
;


\ set-count

: set-count		\ a -- ;
  dup
  index-ptr @
  swap - 5 -
  swap index-buf + c!
;

hex

: bold-end?		\ a -- t/f ;
  begin
    dup c@ bl =
  while
    1+
  repeat
  c@ 9c =
;

decimal


\ move-chr

: move-chr	\ a -- a+1 ; transfer a character
  dup				\
  c@				\ get character
  index-buf index-ptr @ +	\ destination address
  c!				\ store it
  1+				\
  1 index-ptr +!		\ increment pointer
;

: set-addr	\ -- ; set index address
  file-ptr 2@ 4, d+		\ current file position
  index-buf index-ptr @ +	\ destination address
  2!				\ store it
  4 index-ptr +!		\ skip over
;

: >index	\ a -- ; insert into index
  index-ptr @ swap		\ save start of record
  1 index-ptr +!
  begin			\
    dup bold-end?		\ at end of header ?
    dup end-of-buf @ =		\ or at end of buffer
    or not			\
  while			\
    move-chr			\ transfer character to buffer
  repeat			\
  drop				\
  set-addr			\ set index address
  set-count			\ set count byte
;

: process-it		\ -- ; generate index
  end-of-buf @ 20 - 0		\ all buffer
  ?do				\
    buffer i +			\ current position in buffer
    'header-code		\ address of header string
    5 s=			\ compare
    if				\ if a header
      buffer i + 5 +		\ place in index
      >index			\
    endif			\
  1, file-ptr d+!		\ increment file pointer
  loop				\
;

: save-it	\ -- ; save index buffer to disk
  index-buf			\ start of buffer
  index-ptr @			\ length of buffer
  index-pcb handle		\ file's handle
  write-path			\ write to disk
  if ." write error" bye endif	\ if can't write, error & abort
  drop
;

: close-files	\ -- ; close all files
  gloss-pcb handle		\ close input file
  close-path			\
  index-pcb handle		\ close output file
  close-path			\
;

: ?read-error		\ status -- ; abort if a read error
  if ." read error" bye endif
;

: 2+!		\ n a -- ; 32-bit +!
  >r
  s>d
  r@ 2@
  d+
  r>
  2!
;

: load-2k	\ -- ; load 2k into buffer
  buffer			\ buffer addr
  load-size			\ amount to load
  gloss-pcb handle		\ glossary's handle
  read-path			\ read it
  ?read-error			\ a read error ?
  dup end-of-buf !		\ save the amount read
  20 - end-of-gloss 2+!	\ update current length
  ." ."			\ show its doing something
  0 index-ptr !		\ reset pointer
  buffer load-size upper	\ convert buffer to upper case
;

: ?opened-ok		\ pcb-addr status -- ;
  0<>				\ if file not open
  if				\
    ." cannot open " .pcb	\ display error
    bye			\ abort
  else				\
    drop			\ otherwise drop pcb-addr
  endif
;


\ open-files

: open-files		\ -- ; open input and output files
  bl word dup >r			\ get glossary name
  gloss-pcb set-pathname		\ set pathname in pcb
  gloss-pcb dup			\ glossary's pcb
  open-path-pcb			\ try to open it
  ?opened-ok				\ is it open ?
  cr cr ." Loading: "			\ print title
  r> dup dup c@ 4 - swap c!		\
  index-pcb set-pathname		\ set index's pathname
  "" .idx" index-pcb set-ext		\ set index extension
  index-pcb dup			\ index's pcb
  create-path-pcb			\ create it
  ?opened-ok				\ is it open ?
;

: save-length		\ -- ; save length of glossary at end of index
  end-of-gloss 
  4
  index-pcb handle
  write-path
  2drop
;

: index-gloss		\ -- ; genarate index for glossary
  open-files			\ open input & output files
  begin			\
    file-ptr 2@		\ offset into file
    gloss-pcb handle		\ offset = back 50 bytes
    move-file-ptr drop		\ move file pointer back a bit
    load-2k			\ load next 2k into buffer
    process-it			\ generate bit of index
    save-it			\ save index
    at-end?			\ at end of file ?
  until			\
  20 end-of-gloss 2+!
  save-length
  close-files
  2drop
  bye
;
